<?php

namespace Drupal\external_entities_field_processors\Plugin\ExternalEntities\FieldMapper;

use Drupal\external_entities_field_processors\FieldMapProcessor\FieldMapProcessorManager;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\external_entities\Plugin\ExternalEntities\FieldMapper\SimpleFieldMapper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A field mapper based on SimpleFieldMapper with processors fields added.
 *
 * @FieldMapper(
 *   id = "field_map_processors_mapper",
 *   label = @Translation("Fields processors mapper"),
 *   description = @Translation("Maps entity field to raw data and allow to
 *     select diferent processors for each field property.")
 * )
 *
 * @package Drupal\external_entities\Plugin\ExternalEntities\FieldMapper
 */
class FieldMapProcessorsMapper extends SimpleFieldMapper {

  /**
   * The field map processor manager.
   *
   * @var \Drupal\external_entities_field_processors\FieldMapProcessor\FieldMapProcessorManager
   */
  protected FieldMapProcessorManager $fieldMapProcessorManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->fieldMapProcessorManager = $container->get('plugin.manager.external_entities_field_processors.field_map_processor_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form += parent::buildConfigurationForm($form, $form_state);
    $form['field_mappings']['#type'] = 'container';
    return $form;
  }

  /**
   * Build a form element for configuring a field mapping.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   *
   * @return array
   *   The form element for configuring the field.
   */
  protected function buildFieldMappingElement(FieldDefinitionInterface $field_definition): array {
    $element = [];

    $property_definitions = $this->getMappableFieldProperties($field_definition);
    foreach ($property_definitions as $property_name => $property_definition) {
      $field_name = $field_definition->getName();
      $element[$property_name] = $this->buildFieldPropertyMappingElement($field_definition, $property_name, $property_definition);
      $element[$property_name . '_processor'] = $this->buildFieldProcessorSelector($field_name, $property_name, $field_definition->getLabel() . ' » ' . $property_definition->getLabel());
    }

    return $element;
  }

  /**
   * Return the field property processor select element.
   *
   * @param string $field_name
   *   The field name.
   * @param string $property_name
   *   The field property name.
   * @param string $label
   *   The selector label.
   *
   * @return array
   *   The form element for selecting the field property processor.
   */
  protected function buildFieldProcessorSelector(string $field_name, string $property_name, string $label = ''): array {
    $processors = $this->fieldMapProcessorManager->getDefinitions();
    $options = ['' => '- None -'];
    foreach ($processors as $processor_id => $processor) {
      $options[$processor_id] = $processor['label'];
    }
    $element = [
      '#type' => 'select',
      '#title' => $this->t('@property_name Processor', ['@property_name' => $label]),
      '#options' => $options,
      '#default_value' => $this->getFieldPropertyProcessor($field_name, $property_name),

    ];
    return $element;
  }

  /**
   * Return the current field property processor.
   *
   * @param string $field_name
   *   The field name.
   * @param string $property_name
   *   The field property name.
   *
   * @return string
   *   The field property processor id.
   */
  protected function getFieldPropertyProcessor(string $field_name, string $property_name): string {
    $field_mappings = $this->getFieldMappings();

    if (empty($field_mappings[$field_name])) {
      return '';
    }

    if (empty($property_name) && isset($field_mappings[$field_name . '_processor'])) {
      return $field_mappings[$field_name . '_processor'];
    }

    return $field_mappings[$field_name][$property_name . '_processor'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  protected function processPropertyValue(FieldDefinitionInterface $field_definition, $property_name, $property_value) {
    $property_value = parent::processPropertyValue($field_definition, $property_name, $property_value);
    $processor_id = $this->getFieldPropertyProcessor($field_definition->getName(), $property_name);
    if (empty($processor_id)) {
      return $property_value;
    }
    $processor = $this->fieldMapProcessorManager->createInstance($processor_id);
    return $processor->processData($property_value, $field_definition);
  }

}
